package re;

import java.util.*;
import java.util.Map.Entry;

import re.Util;

public class NFA {

	protected int acceptState = -1;
	
	protected Vector< Hashtable<Character,Vector<Integer>>> states =
		new Vector<Hashtable<Character,Vector<Integer>>>();
	
	public NFA() {
		states.add(new Hashtable<Character,Vector<Integer>>());
	}
	
	protected Hashtable<Character,Vector<Integer>> tableForState(int i) {
		Util.assertTrue(0 <= i && i < states.size(), "Bad State: " + i);
		return states.get(i);
	}
	
	public int getStartState() {
		return 0;
	}
	
	public void setAcceptState(int state) {
		Util.assertTrue(0 <= state && state < states.size(), "Bad State: " + state);
		acceptState = state;
	}
	
	/*
	 * 'a <= c <= 'z', or '@' for epsilon
	 */
	public void addTransition(int start, int end, char c) {
		Util.assertTrue(0 <= start && start < states.size(), "Bad State: " + start);
		Util.assertTrue(0 <= end && end < states.size(), "Bad State: " + end);
		Util.assertTrue('a' <= c && c <= 'z' || c == '@', "Bad symbol: '" + c + "'");
		Hashtable<Character,Vector<Integer>> t = tableForState(start);
		Vector<Integer> v = t.get(c);
		if (v == null) {
			v = new Vector<Integer>();
			t.put(c,v);
		}
		v.add(end);
	}
	
	public int newState() {
		int n = states.size();
		states.add(new Hashtable<Character,Vector<Integer>>());
		return n;
	}

	public String toString() {
		String result = "";
		Util.assertTrue(acceptState != -1, "No accept state specified");
		result += states.size() + "\n";
		result += acceptState + "\n";
		for (int i = 0; i < states.size(); i++) {
			Hashtable<Character,Vector<Integer>> t = states.get(i);
			result += i + " ";
			for (Entry<Character,Vector<Integer>> e : t.entrySet()) {
				result += (e.getKey() + ":(");
				boolean b = false;
				for (int j : e.getValue()) {
					if (b) { result += (","); }
					b = true;
					result += "" + j;
				}
				result += ") ";
			}
			result += "\n";
		}
		return result;
	}
	
	public String toDot() {
		String result = "digraph G {\n    rankdir=LR;\n";
		for (int i = 0; i < states.size(); i++) {
			if (acceptState == i) {
				result += "  S" + i + "[label=\"" + i + "\", peripheries=2,regular=true];\n";
			} else { 
				result += "  S" + i + "[label=\"" + i + "\",regular=true];\n";
			}
			
		}
		
		for (int i = 0; i < states.size(); i++) {
			Hashtable<Character,Vector<Integer>> t = states.get(i);
			for (Entry<Character,Vector<Integer>> e : t.entrySet()) {
				char c = e.getKey();
				for (int j : e.getValue()) {
					result += "  S" + i + " -> S" + j + (c == 0 ? "" : "[label=\"" + c + "\"]") + ";\n";
				}
			}
		}
		result += "}\n";
		return result;
	}
}
