package re.ast

abstract class RENode { 
  	/** Accepts a visitor object as part of the visitor pattern.
	 * @param visitor A visitor.
	 */
	def accept(visitor : Visitor);
	
	/** Accepts a propagating visitor parameterized by two types.
	 * 
	 * @param <DownType> The type of the object holding the context.
	 * @param <UpType> The type of the result object.
	 * @param visitor A propagating visitor.
	 * @param context An object holding context information.
	 * @return The result of visiting this node.
	 */
	def accept[DownType, UpType](visitor : PropagatingVisitor[DownType, UpType], 
								 context : DownType) : UpType;

  
}

